using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Effects;
using Jypeli.Widgets;
using Color = Jypeli.Color;
using Image = Jypeli.Image;
using Timer = Jypeli.Timer;
using Vector = Jypeli.Vector;

namespace Peli
{
    /// @author Häyrinen Viljami, Poikolainen Aaron, Kaikosuo Paavo
    /// @version 1.0.00
    public class Peli : PhysicsGame
    {
        /// <summary>
        /// Ladataan kuvat ja luodaan niille oliot
        /// </summary>
        Image autoSprite = LoadImage("auto.png");
        Image autoTie = LoadImage("Autotie.png");
        Image varjo = LoadImage("Varjo.png");
        Image tausta = LoadImage("tausta.png");
        Image taukovalikonTausta = LoadImage("taukovalikonTausta.png");
        Image menunTausta = LoadImage("menuBackground.png");
        Image sekuntikello = LoadImage("ajastin.png");
        Image puu = LoadImage("Puu.png");
        Image kivi = LoadImage("tuukka18v.png");
        Image maaliviivakuva = LoadImage("maaliviiva.png");

        /// <summary>
        /// Täällä on erilaisia muuttujia ja olioita määritelty
        /// </summary>
        private Image taustapaalla = LoadImage("taustapaalla.png");
        private Automobile auto; // Auto-objekti
        public GameObject autoVarjo; // Auton varjo
        public Smoke savu; // Savuefekti
        public bool ajastinKaynnistetty; // Onko ajastin käynnistetty
        private PhysicsObject maaliviiva; // Maaliviiva
        private double aikaaKulunut; // Kulunut aika kisan käynnistyksestä
        private ScoreList topList; // Pistelista
        private bool maalissa; // Onko auto maalissa
        int LapaistutCheckpointit; // Läpäistyjen tarkastuspisteiden määrä
        private Timer aikalaskuri; //Ajanlaskija ajastinta varten

        /// <summary>
        /// Checkpointtien positio tietoja listoissa
        /// </summary>
        double[] xpistec =
        [
            -502, 158, 322, 1160, 630, -90, -609, -866, 715, 302, -334, 12, 899, -1133, 156, -152
        ]; // Tarkastuspisteiden x-koordinaatit

        double[] ypistec =
        [
            -897, -1382, -1962, -728, 11, -368, 334, 1104, 670, 1343, 1730, 1989, -1438, 599, 485, 1340
        ]; // Tarkastuspisteiden y-koordinaatit

        int[] kulmac = [0, 80, 0, 90, 0, 125, 0, 0, 90, 0, 90, 0, 60, 90, -27, 141]; // Tarkastuspisteiden kulmat
        
        /// <summary>
        /// Erilaisia tarkistinmuuttujia ja UI asioita
        /// </summary>
        private bool alkuvalikosta; // Onko peli alkuvalikossa
        List<Label> valikonKohdat; // Valikon kohdat
        List<GameObject> valikonKuvat; // Valikon kuvat
        GameObject valikonTausta; // Valikon tausta
        Label taukoOtsikko; // Taukovalikon otsikko
        private bool kolaroitu; // Onko auto kolaroinut
        Label aikanaytto; // Ajan näyttö
        GameObject ajastinKuva; // Ajastimen kuva

        /// <summary>
        /// Update metodi
        /// </summary>
        /// <param name="time"></param>
        protected override void Update(Time time)
        {
            if (autoVarjo != null)
            {
                autoVarjo.Angle = auto.Angle; // Päivitetään varjon kulma
                autoVarjo.X = auto.X + 10; // Päivitetään varjon x-koordinaatti
                autoVarjo.Y = auto.Y - 10; // Päivitetään varjon y-koordinaatti
            }

            if (savu != null)
            {
                // Jos auto liikkuu ja kääntyy, näytetään savu
                if ((Keyboard.IsKeyDown(Key.A) || Keyboard.IsKeyDown(Key.D) || Keyboard.IsKeyDown(Key.Left) ||
                     Keyboard.IsKeyDown(Key.Right)) && auto.Velocity.Magnitude > 200)
                {
                    savu.MinLifetime = 1;
                    savu.MaxLifetime = 1.5;
                }
                else
                {
                    savu.MinLifetime = 0;
                    savu.MaxLifetime = 0;
                }
            }
            base.Update(time);
        }

        
        /// <summary>
        /// Alustetaan auto-objekti ja sille ominaisuuksia
        /// </summary>
        public void Auto()
        {
            auto = new Automobile(120, 60); // Leveys 120, korkeus 60
            auto.Image = autoSprite;
            this.Add(auto, 1);
            auto.Mass = 10; //massa
            auto.LinearDamping = 1;
            auto.CanRotate = true;
            auto.AngularDamping = 1;
            auto.Acceleration = 300;
            auto.Maneuverability = Angle.FromDegrees(3);
            auto.BrakeDeceleration = 300;
        }

        
        /// <summary>
        /// Peli alkaa tästä metodista
        /// </summary>
        public override void Begin()
        {
            ClearAll();
            IsFullScreen = true;
            alkuvalikosta = true;
            topList = new ScoreList(10, true, 10000);
            DataStorage.TryLoad(topList, "pisteet.xml");
            Keyboard.Listen(Key.F11, ButtonState.Pressed, F11, "Fullscreen"); 
            Valikko();
            Level.Background.Color = new Color(160, 160, 160);
            Level.Size = Screen.Size;
            Level.Background.Image = menunTausta;
            Level.Background.Image.Scaling = ImageScaling.Nearest;
            Level.Background.ScaleToLevelByWidth();
            Level.Background.ScaleToLevelByHeight();
            MediaPlayer.Play("taustaMusiikki.wav");
            MediaPlayer.IsRepeating = true;
        }
        
        
        /// <summary>
        /// Aliohjelma pelin aloittamiselle.
        /// </summary>
        public void AloitaPeli()
        {
            ClearAll();
            Level.Background.Color = new Color(48, 126, 40); //Taustaväri
            Auto();
            autoTie.Scaling = ImageScaling.Nearest;
            ajastinKaynnistetty = false;
            Level.Background.Image = autoTie;
            Camera.Follow(auto); // Asetetaan kamera seuraamaan autoa
            auto.Position = new Vector(-804.886, -1790.033);
            auto.Angle = Angle.FromDegrees(90);
            aikaaKulunut = 0;
            autoVarjo = new GameObject(120, 60);
            autoVarjo.Image = varjo;
            LapaistutCheckpointit = 0;
            this.Add(autoVarjo, -2); //Lisätään varjo peliin
            alkuvalikosta = false;
            Savu(); // Savu autoon
            Esteet(); // Esteet
            Maali(); // Maali
            Checkpointit(); // Tarkastuspisteet
            kolaroitu = false;
            // Kuuntelijat törmäyksille
            AddCollisionHandler(auto, maaliviiva, Maalissa);
            AddCollisionHandler(auto, "rataeste", CollisionHandler.ExplodeObject(100.0, true));
            AddCollisionHandler(auto, "rataeste", Kolari);
            AddCollisionHandler(auto, "checkpointti", Checkpoint);
            // Kuuntelijat peliin WASD ja esc näppäimiä varten
            PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, Pauseta, "Avaa valikon");
            Keyboard.Listen(Key.W, ButtonState.Down, W, "Auto kaasuttaa");
            Keyboard.Listen(Key.S, ButtonState.Down, S, "Auto jarruttaa");
            Keyboard.Listen(Key.A, ButtonState.Down, A, "Auto kääntää vasemmalle");
            Keyboard.Listen(Key.D, ButtonState.Down, D, "Auto kääntää oikealle");
            Keyboard.Listen(Key.Up, ButtonState.Down, W, "Auto kaasuttaa");
            Keyboard.Listen(Key.Down, ButtonState.Down, S, "Auto jarruttaa");
            Keyboard.Listen(Key.Left, ButtonState.Down, A, "Auto kääntää vasemmalle");
            Keyboard.Listen(Key.Right, ButtonState.Down, D, "Auto kääntää oikealle");
            Keyboard.Listen(Key.LeftShift, ButtonState.Down, Shift, "Auto kääntää pakittaa");
            Keyboard.Listen(Key.C, ButtonState.Pressed, C, "Koodrinaatit");
            Keyboard.Listen(Key.R, ButtonState.Pressed, AloitaAlusta, "Restart");
            Keyboard.Listen(Key.F11, ButtonState.Pressed, F11, "Fullscreen");
        }
        
        
        /// <summary>
        /// Aliohjelma alkuvalikon luomista verten
        /// </summary>
        private void AlkuValikko()
        {
            ClearAll();
            alkuvalikosta = true;
            Valikko();
            Level.Background.Color = new Color(160, 160, 160);
            Level.Background.Image = menunTausta;
            Level.Background.Image.Scaling = ImageScaling.Nearest;
            Level.Background.ScaleToLevelByWidth();
            Level.Background.ScaleToLevelByHeight();
        }
        
        
        /// <summary>
        /// Pelin alkuvalikko. Luo ja näyttää.
        /// </summary>
        private void Valikko()
        {
            tausta.Scaling = ImageScaling.Nearest;
            Label otsikko = new Label("Tervetuloa pelaamaan Dumb Ways to Drive peliä!"); // Luodaan otsikko
            otsikko.Position = new Vector(480, 190); // Otsikko on hieman valikonkohtien yläpuolella
            otsikko.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            otsikko.TextColor = Color.White;
            Add(otsikko);

            valikonKohdat = new List<Label>(); // Alustetaan lista, johon valikon kohdat tulevat

            Label kohta1 = new Label(220, 65, "Aloita uusi peli"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta1.Position = new Vector(480, 90); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta1); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta1.TextColor = Color.White;

            GameObject kohta1go = new GameObject(220, 65);
            kohta1go.Position = kohta1.Position;
            kohta1go.Image = tausta;
            Add(kohta1go);

            Label kohta2 = new Label(220, 65, "Parhaat pisteet");
            kohta2.Position = new Vector(480, 0);
            valikonKohdat.Add(kohta2);
            kohta2.TextColor = Color.White;
            GameObject kohta2go = new GameObject(220, 65);
            kohta2go.Position = kohta2.Position;
            kohta2go.Image = tausta;
            Add(kohta2go);

            Label kohta3 = new Label(220, 65, "Kontrollit");
            kohta3.Position = new Vector(480, -90);
            valikonKohdat.Add(kohta3);
            kohta3.TextColor = Color.White;
            GameObject kohta3go = new GameObject(220, 65);
            kohta3go.Position = kohta3.Position;
            kohta3go.Image = tausta;
            Add(kohta3go);
            
            Label kohta4 = new Label(220, 65, "Lopeta peli");
            kohta4.Position = new Vector(480, -180);
            valikonKohdat.Add(kohta4);
            kohta4.TextColor = Color.White;
            GameObject kohta4go = new GameObject(220, 65);
            kohta4go.Position = kohta4.Position;
            kohta4go.Image = tausta;
            Add(kohta4go);
            // Lisätään kaikki luodut kohdat peliin foreach-silmukalla
            foreach (Label valikonKohta in valikonKohdat)
            {
                Add(valikonKohta);
            }

            Mouse.ListenOn(kohta1, MouseButton.Left, ButtonState.Pressed, AloitaPeli, null);
            Mouse.ListenOn(kohta2, MouseButton.Left, ButtonState.Pressed, Leaderboard, null);
            Mouse.ListenOn(kohta3, MouseButton.Left, ButtonState.Pressed, Kontrollit, null);
            Mouse.ListenOn(kohta4, MouseButton.Left, ButtonState.Pressed, Exit, null);


            Mouse.ListenOn(kohta1, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta1go, true);
            Mouse.ListenOn(kohta1, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta1go, false);

            Mouse.ListenOn(kohta2, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta2go, true);
            Mouse.ListenOn(kohta2, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta2go, false);

            Mouse.ListenOn(kohta3, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta3go, true);
            Mouse.ListenOn(kohta3, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta3go, false);
            Mouse.ListenOn(kohta4, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta4go, true);
            Mouse.ListenOn(kohta4, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta4go, false);
        }

        
        /// <summary>
        /// Valikko, joka ilmestyy kun pelaaja tauottaa pelin (esc-näppäin)
        /// </summary>
        public void taukoValikko()
        {
            valikonTausta = new GameObject(224, 444);
            valikonTausta.Position = new Vector(Camera.X, Camera.Y + 25);
            valikonTausta.Image = taukovalikonTausta;
            Add(valikonTausta, 2);

            valikonKohdat = new List<Label>(); // Alustetaan lista, johon valikon kohdat tulevat
            valikonKuvat = new List<GameObject>();
            Label kohta1 = new Label(174, 54, "Jatka peliä"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta1.Position = new Vector(Camera.Position.X, Camera.Position.Y + 150); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta1); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta1.TextColor = Color.White;

            GameObject kohta1go = new GameObject(174, 54);
            kohta1go.Position = kohta1.Position;
            kohta1go.Image = tausta;
            valikonKuvat.Add(kohta1go);

            taukoOtsikko = new Label("Taukovalikko"); // Luodaan otsikko
            taukoOtsikko.Position = new Vector(kohta1.X, kohta1.Y + 60); // Otsikko on hieman valikonkohtien yläpuolella
            taukoOtsikko.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            taukoOtsikko.TextColor = Color.White;
            Add(taukoOtsikko, 2);

            Label kohta2 = new Label(174, 54, "Pelaa uudelleen");
            kohta2.Position = new Vector(Camera.Position.X, Camera.Position.Y + 75);
            valikonKohdat.Add(kohta2);
            kohta2.TextColor = Color.White;
            GameObject kohta2go = new GameObject(174, 54);
            kohta2go.Position = kohta2.Position;
            kohta2go.Image = tausta;
            valikonKuvat.Add(kohta2go);

            Label kohta3 = new Label(174, 54, "Alkuvalikko");
            kohta3.Position = new Vector(Camera.Position.X, Camera.Position.Y);
            valikonKohdat.Add(kohta3);
            kohta3.TextColor = Color.White;
            GameObject kohta3go = new GameObject(174, 54);
            kohta3go.Position = kohta3.Position;
            kohta3go.Image = tausta;
            valikonKuvat.Add(kohta3go);

            Label kohta4 = new Label(174, 54, "Parhaat ajat");
            kohta4.Position = new Vector(Camera.Position.X, Camera.Position.Y - 75);
            valikonKohdat.Add(kohta4);
            kohta4.TextColor = Color.White;
            GameObject kohta4go = new GameObject(174, 54);
            kohta4go.Position = kohta4.Position;
            kohta4go.Image = tausta;
            valikonKuvat.Add(kohta4go);

            Label kohta5 = new Label(174, 54, "Lopeta peli");
            kohta5.Position = new Vector(Camera.Position.X, Camera.Position.Y - 150);
            valikonKohdat.Add(kohta5);
            kohta5.TextColor = Color.White;
            GameObject kohta5go = new GameObject(174, 54);
            kohta5go.Position = kohta5.Position;
            kohta5go.Image = tausta;
            valikonKuvat.Add(kohta5go);

            // Lisätään kaikki luodut kohdat peliin foreach-silmukalla
            foreach (Label valikonKohta in valikonKohdat)
            {
                Add(valikonKohta, 3);
            }

            foreach (GameObject valikonKuva in valikonKuvat)
            {
                Add(valikonKuva, 3);
            }

            Mouse.ListenOn(kohta1, MouseButton.Left, ButtonState.Pressed, Pauseta, null);
            Mouse.ListenOn(kohta2, MouseButton.Left, ButtonState.Pressed, AloitaAlusta, null);
            Mouse.ListenOn(kohta3, MouseButton.Left, ButtonState.Pressed, AlkuValikko, null);
            Mouse.ListenOn(kohta4, MouseButton.Left, ButtonState.Pressed, Leaderboard, null);
            Mouse.ListenOn(kohta5, MouseButton.Left, ButtonState.Pressed, Exit, null);

            Mouse.ListenOn(kohta1, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta1go, true);
            Mouse.ListenOn(kohta1, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta1go, false);

            Mouse.ListenOn(kohta2, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta2go, true);
            Mouse.ListenOn(kohta2, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta2go, false);

            Mouse.ListenOn(kohta3, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta3go, true);
            Mouse.ListenOn(kohta3, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta3go, false);

            Mouse.ListenOn(kohta4, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta4go, true);
            Mouse.ListenOn(kohta4, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta4go, false);

            Mouse.ListenOn(kohta5, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta5go, true);
            Mouse.ListenOn(kohta5, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen,
                null, kohta5go, false);
        }

        
        /// <summary>
        /// Valikko, joka ilmestyy kun pelaaja kuolee (kolaroi)
        /// </summary>
        public void kuolemaValikko()
        {
            valikonTausta = new GameObject(224, 350);
            valikonTausta.Position = new Vector(Camera.X, Camera.Y + 25);
            valikonTausta.Image = taukovalikonTausta;
            Add(valikonTausta, 2);


            valikonKohdat = new List<Label>(); // Alustetaan lista, johon valikon kohdat tulevat
            valikonKuvat = new List<GameObject>();
            Label kohta1 = new Label(200, 60,
                    "Pelaa uudelleen"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta1.Position = new Vector(Camera.Position.X, Camera.Position.Y + 85); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta1); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta1.TextColor = Color.White;

            GameObject kohta1go = new GameObject(200, 60);
            kohta1go.Position = kohta1.Position;
            kohta1go.Image = tausta;
            valikonKuvat.Add(kohta1go);

            taukoOtsikko = new Label("Ajoit kolarin... ei yllätä"); // Luodaan otsikko
            taukoOtsikko.Position = new Vector(kohta1.X, kohta1.Y + 60); // Otsikko on hieman valikonkohtien yläpuolella
            taukoOtsikko.Font = new Font(25, true); // Otsikon teksti on suurempi ja boldattu
            taukoOtsikko.TextColor = Color.White;
            Add(taukoOtsikko, 2);

            Label kohta2 = new Label(200, 50, "Alkuvalikko");
            kohta2.Position = new Vector(Camera.Position.X, Camera.Position.Y);
            valikonKohdat.Add(kohta2);
            kohta2.TextColor = Color.White;

            GameObject kohta2go = new GameObject(200, 60);
            kohta2go.Position = kohta2.Position;
            kohta2go.Image = tausta;
            valikonKuvat.Add(kohta2go);

            Label kohta3 = new Label(200, 60, "Lopeta peli");
            kohta3.Position = new Vector(Camera.Position.X, Camera.Position.Y - 85);
            valikonKohdat.Add(kohta3);
            kohta3.TextColor = Color.White;

            GameObject kohta3go = new GameObject(200, 60);
            kohta3go.Position = kohta3.Position;
            kohta3go.Image = tausta;
            valikonKuvat.Add(kohta3go);

            // Lisätään kaikki luodut kohdat peliin foreach-silmukalla
            foreach (Label valikonKohta in valikonKohdat)
            {
                Add(valikonKohta, 3);
            }

            foreach (GameObject valikonKuva in valikonKuvat)
            {
                Add(valikonKuva, 3);
            }

            Mouse.ListenOn(kohta1, MouseButton.Left, ButtonState.Pressed, AloitaAlusta, null);
            Mouse.ListenOn(kohta2, MouseButton.Left, ButtonState.Pressed, AlkuValikko, null);
            Mouse.ListenOn(kohta3, MouseButton.Left, ButtonState.Pressed, Exit, null);

            Mouse.ListenOn(kohta1, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta1go, true);
            Mouse.ListenOn(kohta1, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta1go, false);

            Mouse.ListenOn(kohta2, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta2go, true);
            Mouse.ListenOn(kohta2, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta2go, false);

            Mouse.ListenOn(kohta3, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta3go, true);
            Mouse.ListenOn(kohta3, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, kohta3go, false);

        }
        
        
        /// <summary>
        /// Kontrollimenu
        /// </summary>
        private void Kontrollit()
        {
            ClearAll();
            Level.Background.Color = new Color(160, 160, 160);
            Label otsikko = new Label("Pelin kontrollit"); // Luodaan otsikko
            otsikko.Position = new Vector(0, 200); 
            otsikko.Font = new Font(50, true); // Otsikon teksti on suurempi ja boldattu
            otsikko.TextColor = Color.White;
            Add(otsikko);

            valikonKohdat = new List<Label>(); // Alustetaan lista, johon valikon kohdat tulevat
            
            Label kohta1 = new Label(600, 65, "W/Ylä nuoli  -  Auto kaasuttaa"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta1.Position = new Vector(0, 80); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta1); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta1.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            kohta1.TextColor = Color.White;
            
            Label kohta2 = new Label(600, 65, "S/Ala nuoli  -  Auto jarruttaa"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta2.Position = new Vector(0, 0); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta2); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta2.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            kohta2.TextColor = Color.White;
            
            Label kohta3 = new Label(600, 65, "A/Vasen nuoli  -  Auto kääntää vasemmalle"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta3.Position = new Vector(0, 40); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta3); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta3.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            kohta3.TextColor = Color.White;
            
            Label kohta4 = new Label(600, 65, "D/Oikea nuoli  -  Auto kääntää oikealle"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta4.Position = new Vector(0, -40); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta4); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta4.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            kohta4.TextColor = Color.White;
            
            Label kohta5 = new Label(600, 65, "Shift - Auto pakittaa"); // Luodaan uusi Label-olio, joka toimii uuden pelin aloituskohtana
            kohta5.Position = new Vector(0, -80); // Asetetaan valikon ensimmäinen kohta hieman kentän keskikohdan yläpuolelle
            valikonKohdat.Add(kohta5); // Lisätään luotu valikon kohta listaan jossa kohtia säilytetään
            kohta5.Font = new Font(30, true); // Otsikon teksti on suurempi ja boldattu
            kohta5.TextColor = Color.White;
            
            Label nappi = new Label(220, 65, "Palaa alkuvalikkoon");
            nappi.Position = new Vector(0, -200);
            valikonKohdat.Add(nappi);
            nappi.TextColor = Color.White;
            GameObject nappihitbox = new GameObject(220, 65);
            nappihitbox.Position = nappi.Position;
            nappihitbox.Image = tausta;
            Add(nappihitbox);
            
            foreach (Label valikonKohta in valikonKohdat)
            {
                Add(valikonKohta, 3);
            }
            Mouse.ListenOn(nappi, MouseButton.Left, ButtonState.Pressed, AlkuValikko, null);
            Mouse.ListenOn(nappi, HoverState.Enter, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, nappihitbox, true);
            Mouse.ListenOn(nappi, HoverState.Exit, MouseButton.None, ButtonState.Irrelevant, ValikossaLiikkuminen, null, nappihitbox, false);
        }
        
        
        /// <summary>
        /// Aliohjelma, joka palauttaa pelaajan takaisin pelin alkuruutuun.
        /// </summary>
        /// <param name="sender"></param>
        private void takaisinAlkuvalikkoon(Window sender)
        {
            AlkuValikko();
        }
        
        
        /// <summary>
        /// Pausetuksen logiikka
        /// </summary>
        private void Pauseta()
        {
            if (kolaroitu == false)
            {
                if (IsPaused)
                {
                    foreach (Label valikonKohta in valikonKohdat)
                    {
                        Remove(valikonKohta);
                    }

                    foreach (GameObject valikonKuva in valikonKuvat)
                    {
                        Remove(valikonKuva);
                    }

                    Remove(valikonTausta);
                    Remove(taukoOtsikko);
                    IsPaused = false;
                    if (aikalaskuri != null)
                    {
                        aikalaskuri.Start();
                    }
                }
                else
                {
                    if (aikalaskuri != null)
                    {
                        aikalaskuri.Pause();
                    }

                    IsPaused = true;
                    taukoValikko();
                }
            }
        }

        
        /// <summary>
        /// Valikossa liikkumisen logiikka, jonka avulla napit pystyvät vaihtamaan väriään.
        /// </summary>
        /// <param name="kohta">Kohta(nappi), jonka päällä hiiri on</param>
        /// <param name="paalla">boolean arvo sille onko hiiri napin päällä vai ei</param>
        void ValikossaLiikkuminen(GameObject kohta, bool paalla)
        {
            if (paalla)
            {
                kohta.Image = taustapaalla;
            }
            else
            {
                kohta.Image = tausta;
            }
        }

        
        /// <summary>
        /// Auton osuman maaliviivaan kuuntelija.
        /// </summary>
        /// <param name="tormaaja">Auto</param>
        /// <param name="kohde">Maaliviiva</param>
        private void Maalissa(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            auto.Velocity = Vector.Zero;
            aikalaskuri.Stop();
            aikaaKulunut = aikalaskuri.SecondCounter.Value;
            maalissa = true;
            Leaderboard();
        }

        
        /// <summary>
        /// Maalin luomis aliohjelma. Luo maalin.
        /// </summary>
        private void Maali()
        {
            maaliviiva = new PhysicsObject(50, 200);
            maaliviiva.IgnoresCollisionResponse = true;
            maaliviiva.Position = new Vector(950, 1793);
            maaliviiva.Shape = Shape.Rectangle;
            maaliviiva.Image = maaliviivakuva;
            Add(maaliviiva);
        }

        
        /// <summary>
        ///Luodaan ja näytetään pistelista
        /// </summary>
        private void Leaderboard()
        {
            ClearAll();
            if (maalissa && LapaistutCheckpointit == kulmac.Length)
            {
                aikaaKulunut = Math.Round(aikaaKulunut, 2);
                HighScoreWindow topIkkuna = new HighScoreWindow("Parhaat ajat",
                    "Onneksi olkoon, pääsit listalle ajalla %p! Syötä nimesi:", topList, aikaaKulunut);

                topIkkuna.Closed += (_) =>
                {
                    AloitaAlusta();
                };
                Add(topIkkuna);
            }
            else if (LapaistutCheckpointit == kulmac.Length && maalissa == false)
            {
                HighScoreWindow topIkkuna = new HighScoreWindow("Parhaat ajat", topList);
                topIkkuna.Closed += (_) =>
                {
                    AloitaAlusta();
                };
                Add(topIkkuna);
            }
            else if (maalissa && LapaistutCheckpointit != kulmac.Length)
            {
                HighScoreWindow topIkkuna = new HighScoreWindow("Koitahan ensi kerralla pysyä tiellä...", topList);
                topIkkuna.Closed += (_) =>
                {
                    AloitaAlusta();
                };
                Add(topIkkuna);
            }
            else if (alkuvalikosta)
            {
                HighScoreWindow topIkkuna = new HighScoreWindow("Parhaat ajat", topList);
                topIkkuna.Closed += takaisinAlkuvalikkoon;
                Add(topIkkuna);
            }
            else
            {
                HighScoreWindow topIkkuna = new HighScoreWindow("Parhaat ajat", topList);
                topIkkuna.Closed += (_) =>
                {
                    AloitaAlusta();
                };
                Add(topIkkuna);
            }

        }
        
        
        /// <summary>
        /// Aliohjelma joka luo aikalaskurin
        /// </summary>
        void LuoAikalaskuri()
        {
            aikalaskuri = new Timer();
            aikalaskuri.Start();
            aikanaytto = new Label(200, 60);
            aikanaytto.X = Screen.Right - 50;
            aikanaytto.Y = Screen.Top - 50;
            aikanaytto.TextColor = Color.Black;
            aikanaytto.Color = new Color(160, 160, 160);
            aikanaytto.DecimalPlaces = 2;
            aikanaytto.BindTo(aikalaskuri.SecondCounter);
            Add(aikanaytto);
            ajastinKuva = new GameObject(70, 70);
            ajastinKuva.Image = sekuntikello;
            aikanaytto.Add(ajastinKuva);
            ajastinKuva.X = aikanaytto.X - 50;
            ajastinKuva.Y = aikanaytto.Y;
        }

        
        /// <summary>
        /// Checkpointtien luomis aliohjelma. Luo checkpointit
        /// </summary>
        private void Checkpointit() //Taulukkoa käytetään checkpointtien luomiseen useampaan kohtaan.
        {
            int n = xpistec.Length;
            PhysicsObject[] Checkpoint = new PhysicsObject[n];
            for (int i = 0; i < n; i++)
            {
                int Asteet = kulmac[i];
                Checkpoint[i] = new PhysicsObject(50, 230);
                Checkpoint[i].Position = new Vector(xpistec[i], ypistec[i]);
                Checkpoint[i].IgnoresCollisionResponse = true;
                Checkpoint[i].IgnoresExplosions = true;
                Checkpoint[i].Shape = Shape.Rectangle;
                Checkpoint[i].Color = Color.LightGreen;
                Checkpoint[i].Tag = "checkpointti";
                Checkpoint[i].Angle = Angle.FromDegrees(Asteet);
                Checkpoint[i].IsVisible = false;
                Add(Checkpoint[i]);
            }
        }

        
        /// <summary>
        /// Auton checkpointtien läpi kulkemisen käsittelijä
        /// </summary>
        /// <param name="tormaaja">Auto</param>
        /// <param name="kohde">Checkpointti(Näkymätön)</param>
        private void Checkpoint(PhysicsObject tormaaja, PhysicsObject kohde)
        {
            LapaistutCheckpointit += 1;
            kohde.Destroy();
        }

        
        /// <summary>
        /// Esteiden luoja aliohjelma. Luo esteet.
        /// </summary>
        private void Esteet() //Luodaan useita esteitä taulukoiden avulla
        {
            double[] xpiste =
            [
                -452.6, -891, -936.7, 919.2, 330.2, -320.4, -103.9, 968.6, -479.4, -650.5, 366.7, -20.6, 277.7, 162.3,
                1315.9, 668.3, 672.3, 134.4, -254.4, -492, -499.8, -934, -1291
            ];
            double[] ypiste =
            [
                515.8, 903.2, 1315.4, 608.2, 1119.4, 2316, 1821.8, 1981.4, -1064.4, -746.6, -1278, -1323.9, -1743.8,
                -2073.9, -716.9, 192.2, -216.2, -283.5, -471.7, 128.1, 1623.9, 517, 852
            ];
            int n = xpiste.Length;
            PhysicsObject[] Este = new PhysicsObject[n];
            for (int i = 0; i < n; i++)
            {
                Este[i] = new PhysicsObject(50, 50);
                Este[i].Position = new Vector(xpiste[i], ypiste[i]);
                Este[i].IgnoresCollisionResponse = true;
                Este[i].IgnoresExplosions = true;
                Este[i].Shape = Shape.Circle;
                Este[i].Color = Color.Red;
                Este[i].Tag = "rataeste";
                Este[i].IsVisible = false;
                Add(Este[i]);
                Image kuva = RandomGen.SelectOne(puu, kivi);
                GameObject kuvapallolle = new GameObject(100, 100);
                kuvapallolle.Image = kuva;
                kuvapallolle.Position = Este[i].Position;
                Add(kuvapallolle);
            }

        }

        
        /// <summary>
        /// Auton törmäyksien esteisiin käsittelijä
        /// </summary>
        /// <param name="tormaaja">Tässä tapauksessa auto</param>
        /// <param name="kohde">Kohde johon törmätään eli puu tai kivi</param>
        private void Kolari(PhysicsObject tormaaja, PhysicsObject kohde) //Aliohjelma, joka suoritetaan kolarin sattuessa.
        {
            kolaroitu = true;
            autoVarjo.Destroy();
            kuolemaValikko();
            aikalaskuri.Stop();
        }

        
        /// <summary>
        /// Aliohjelma, joka käsittelee pelin alusta aloittamisen
        /// </summary>
        private void AloitaAlusta()
        {
            DataStorage.Save(topList, "pisteet.xml");
            maalissa = false;
            ClearAll();
            AloitaPeli();
        }
        
        
        /// <summary>
        /// Auton savun aliohjelma. Luo ja lisää sen autoon.
        /// </summary>
        private void Savu()
        {
            savu = new Smoke();
            savu.MinLifetime = 1;
            savu.MaxLifetime = 1.5;
            savu.MinScale = 50;
            savu.MaxScale = 200;
            savu.AlphaAmount = 0.9;
            savu.Position = new Vector(auto.X, auto.Y - 45);
            savu.Angle = auto.Angle - Angle.FromDegrees(180);
            Add(savu, -1);
            auto.Add(savu);
        }

        
        /// <summary>
        /// Kokoruututila päälle/pois
        /// </summary>
        private void F11()
        {
            if (IsFullScreen)
            {
                IsFullScreen = false;
            }
            else
            {
                IsFullScreen = true;
            }
        }

        
        /// <summary>
        /// W-näppäimen käsittely. Kiihdyttää autoa.
        /// </summary>
        private void W() //Eteen liikkuminen
        {
            if (IsPaused == false)
            {
                if ((Keyboard.IsKeyDown(Key.W) && Keyboard.IsKeyDown(Key.Up) == false) ||
                    (Keyboard.IsKeyDown(Key.W) == false && Keyboard.IsKeyDown(Key.Up)))
                {
                    auto.TopSpeed = 5500; //Huippunopeus eteenpäin
                    auto.Acceleration = 700; //Kiihtyvyys eteenpäin
                    auto.Accelerate(); //Kiihdyttää autoa
                    if (ajastinKaynnistetty == false)
                    {
                        ajastinKaynnistetty = true;
                        LuoAikalaskuri();
                    }
                }
            }
        }

        
        /// <summary>
        /// Jarrutuksen logiikka.
        /// </summary>
        private void S() //Jarrutus
        {
            if (IsPaused == false)
            {
                if ((Keyboard.IsKeyDown(Key.S) && Keyboard.IsKeyDown(Key.Down) == false) ||
                    (Keyboard.IsKeyDown(Key.S) == false && Keyboard.IsKeyDown(Key.Down)))
                {
                    auto.Brake(); //Jarruttaa
                }
            }
        }

        
        // Oikealle kääntyminen
        /// <summary>
        /// D-näppäimen logiikka, käsittelee vasemmalle kääntymistä.
        /// </summary>
        private void D()
        {
            if (IsPaused == false) // Tarkistetaan, onko peli pausella
            {
                // Tarkistetaan, että vain yksi ohjausnäppäin (D tai Right) on painettuna
                if ((Keyboard.IsKeyDown(Key.D) && Keyboard.IsKeyDown(Key.Right) == false) ||
                    (Keyboard.IsKeyDown(Key.D) == false && Keyboard.IsKeyDown(Key.Right)))
                {
                    MuokattuKaantonopeus(); // Päivitetään auton kääntymisnopeus suhteessa nopeuteen

                    // Lasketaan auton suuntavektori
                    Vector suunta = new Vector(Math.Cos(auto.Angle.Radians), Math.Sin(auto.Angle.Radians));

                    // Lasketaan auton liikkeen suunta suhteessa sen eteenpäin suuntautuvaan liikkeeseen
                    double suuntaEteen = auto.Velocity.X * suunta.X + auto.Velocity.Y * suunta.Y;

                    // Jos auto liikkuu taaksepäin eikä Shift-näppäin ole painettuna, käännytään vastakkaiseen suuntaan
                    if (suuntaEteen < 0 && Keyboard.IsShiftDown() == false)
                    {
                        auto.Turn(auto.Maneuverability, 1);
                    }
                    else
                    {
                        auto.Turn(-auto.Maneuverability, 1);
                    }
                }
            }
        }
        
        
        /// <summary>
        /// A-näppäimen logiikka, käsittelee vasemmalle kääntymistä.
        /// </summary>
        private void A()
        {
            if (IsPaused == false) // Tarkistetaan, onko peli pausella
            {
                // Tarkistetaan, että vain yksi ohjausnäppäin (A tai Left) on painettuna
                if ((Keyboard.IsKeyDown(Key.A) && Keyboard.IsKeyDown(Key.Left) == false) ||
                    (Keyboard.IsKeyDown(Key.A) == false && Keyboard.IsKeyDown(Key.Left)))
                {
                    MuokattuKaantonopeus(); // Päivitetään auton kääntymisnopeus suhteessa nopeuteen

                    // Lasketaan auton suuntavektori
                    Vector suunta = new Vector(Math.Cos(auto.Angle.Radians), Math.Sin(auto.Angle.Radians));

                    // Lasketaan auton liikkeen suunta suhteessa sen eteenpäin suuntautuvaan liikkeeseen
                    double suuntaEteen = auto.Velocity.X * suunta.X + auto.Velocity.Y * suunta.Y;

                    // Jos auto liikkuu taaksepäin eikä Shift-näppäin ole painettuna, käännytään vastakkaiseen suuntaan
                    if (suuntaEteen < 0 && Keyboard.IsShiftDown() == false)
                    {
                        auto.Turn(-auto.Maneuverability, 1);
                    }
                    else
                    {
                        auto.Turn(auto.Maneuverability, 1);
                    }
                }
            }
        }
        
        
        /// <summary>
        /// Debuggausta varten. Tulostaa konsoliin auton koordinaatit ja kulman.
        /// </summary>
        private void C() // Tulostetaan auton sijainti ja kulma
        {
            Console.WriteLine(auto.Position);
            Console.WriteLine(auto.Angle.Degrees);
        }
        
        
        /// <summary>
        /// Logiikka auton peruutukselle
        /// </summary>
        private void Shift()
        {
            // Lasketaan auton suuntavektori
            Vector suunta = new Vector(Math.Cos(auto.Angle.Radians), Math.Sin(auto.Angle.Radians));

            // Lasketaan auton liikkeen suunta suhteessa sen eteenpäin suuntautuvaan liikkeeseen
            double suuntaEteen = auto.Velocity.X * suunta.X + auto.Velocity.Y * suunta.Y;

            // Jos auto ei liiku merkittävästi eteenpäin, asetetaan pakitusominaisuudet
            if (suuntaEteen < 1)
            {
                auto.Acceleration = 200; // Asetetaan kiihtyvyys taaksepäin
                auto.TopSpeed = 1000; // Asetetaan huippunopeus taaksepäin
                auto.Reverse(); // Käynnistetään pakitus
            }
        }
        
        
        /// <summary>
        /// Aliohjelma muuttaa auton kääntyvyyttä logaritmifunktion mukaisesti.
        /// </summary>
        private void MuokattuKaantonopeus() // Päivittää auton kääntymisnopeuden suhteessa sen nopeuteen
        {
            if (auto.Velocity.Magnitude > 1) // Jos auton nopeus on riittävä
            {
                // Lasketaan kääntymiskyky logaritmifunktion avulla, jotta suuremmilla nopeuksilla auto kääntyy vähemmän
                auto.Maneuverability = Angle.FromDegrees(Math.Log(auto.Velocity.Magnitude) * 0.3);
            }
            else
            {
                auto.Maneuverability = Angle.FromDegrees(0); // Jos auto on lähes paikallaan, se ei käänny
            }
        }
    }
}